﻿using Common;
using Dapper;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http;
using TGood.Xian.LogManager;
using ThoughtWorks.QRCode.Codec;
using WxPayAPI.Models;
using wxWebAPI.Common;
using wxWebAPI.Filters;
using wxWebAPI.Models;

namespace WxPayAPI
{
    public partial class OrdersController
    {
        PayLogic paylogic = new PayLogic();
        #region 充电相关

        /// <summary>
        /// 查看正在充电的设备
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject NewGetChargingDevice(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<string> deviceCode = chargeLogic.NewGetChargingDevice(userToken);
                if (deviceCode != null)
                {
                    result.Add("code", 1);
                }
                else
                {
                    result.Add("code", 0);
                }
                result.Add("data", JProperty.FromObject(deviceCode));
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetChargingDevice Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 查看正在充电的设备
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <returns></returns>
        [System.Web.Http.HttpGet]
        [TokenFilter]
        public JObject GetChargingDevice(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                String deviceCode = chargeLogic.GetChargingDevice(userToken);
                if (deviceCode != null)
                {
                    result.Add("code", 1);
                }
                else
                {
                    result.Add("code", 0);
                }
                result.Add("data", deviceCode);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetChargingDevice Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 查看充电的设备
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="max"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject NewGetChargingDevices(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<string> deviceCode = chargeLogic.GetChargingDevicesByUserToken(userToken);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(deviceCode));
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetChargingDevice Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }



        /// <summary>
        /// 查看充电的设备
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="max"></param>
        /// <returns></returns>
        [System.Web.Http.HttpGet]
        [TokenFilter]
        public JObject GetChargingDevices(string userToken)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<string> deviceCode = chargeLogic.GetChargingDevicesByUserToken(userToken);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(deviceCode));
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - GetChargingDevice Error,userToken={userToken}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// app侧启动设备，单设备模式
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="deviceCode">设备编码</param>
        /// <param name="showDialog">余额不足是否提示</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
       
        public JObject StartCharge(string userToken, string deviceCode,string nickname)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.StartCharge(userToken, deviceCode,nickname);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - APPStartCharge Error,deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("data", "启动失败，请稍后再试，或选择其他设备");
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// app侧启动设备，多设备模式
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="deviceCode">设备编码</param>
        /// <param name="max">最大可启动数</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject APPStart(string userToken, string deviceCode, int max)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.APPStart(userToken, deviceCode, max);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - APPStartCharge Error,deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 平台侧启动设备
        /// </summary>
        /// <param name="loginID">平台用户id</param>
        /// <param name="token">平台用户token</param>
        /// <param name="deviceCode">设备编码</param>
        /// <param name="max">可启动最大数量</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        
        public JObject PlatformStartCharge(string loginID, string token, string deviceCode, int max = 1000)
        {
            LogEvent.WebAPILoger.Info($"OrdersController - PlatformStartCharge Info, loginID={loginID}, deviceCode={deviceCode}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.PlatformStartCharge(loginID, token, deviceCode, max);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - PlatformStartCharge Error, loginID={loginID}, deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// 平台侧停止设备，单设备模式
        /// </summary>
        /// <param name="loginID">平台用户id</param>
        /// <param name="token">平台用户token</param>
        /// <param name="deviceCode">设备编码</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        
        public JObject PlatformStopCharge(string loginID, string token, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var stopResult = chargeLogic.PlatformStopCharge(loginID, token, deviceCode);

                if (stopResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "此处可能返回一些其他信息");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - PlatformStopCharge Error,loginID={loginID}, deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        ///// <summary>
        ///// 获取充电实时数据
        ///// </summary>
        ///// <param name="userToken">访问令牌</param>
        ///// <param name="deviceCode">设备编码</param>
        ///// <returns></returns>
        //[HttpGet, HttpOptions][TokenFilter]
        //public JObject GetRealData(string userToken, string deviceCode)
        //{
        //    LogEvent.WebAPILoger.Info($"GetRealData  userToken={userToken}, deviceCode={deviceCode}");
        //    JObject result = new JObject();
        //    try
        //    {
        //        var chargeLogic = new ChargeLogic();
        //        result = chargeLogic.GetRealData(userToken, deviceCode);
        //    }
        //    catch (Exception ex)
        //    {
        //        LogEvent.WebAPILoger.Error("GetRealData", ex);
        //        result.Add("code", 0);
        //        result.Add("error", ex.ToString());
        //    }
        //    return result;
        //}

        /// <summary>
        /// 获取设备状态
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns>1 可用;0 其他</returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject NewGetDeviceStatus(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            //默认小程序为领充 支付方式为微信
            EnumAppName app = EnumAppName.领充;
            EnumPayChannel payChannel = EnumPayChannel.微信;
            var chargeLogic = new ChargeLogic();
            try
            {
                //LogEvent.WebAPILoger.Debug($"GetDeviceStatus 进入,userToken={userToken},deviceCode={deviceCode}");

                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                RedisHelper redis5 = new RedisHelper((int)EnumUserRedisNum.Struct);
                if (string.IsNullOrEmpty(userToken))
                {
                    result.Add("code", -10);
                    result.Add("data", "请重新登陆");
                    return result;
                }
                var openid = redis.HashGetString(RedisKey.Token, userToken);


                
                deviceCode = WechatUtility.ShenSiFilter(deviceCode);
                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("data", "请重新登陆");
                    return result;
                }
                if (!deviceCode.Contains("|"))
                {
                    deviceCode = string.Format("{0}|{1}", deviceCode.Substring(0, deviceCode.Length - 2), deviceCode.Substring(deviceCode.Length - 2, 2));
                }

                if (string.IsNullOrEmpty(openid))
                {
                    result.Add("code", -10);
                    result.Add("data", "Token过期");
                    return result;
                }

                string stationid = WechatUtility.GetStationId(deviceCode);
           
                bool ISVIN = CheckVIN(deviceCode, userToken);
                var gunStatus = chargeLogic.GetGunStatus(deviceCode);
                string redisOpenid = "";


                var deviceinfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
                if (deviceinfo != null)
                {
                    redisOpenid = deviceinfo.Openid;
                }
                List<string> openidList = new List<string>();
                List<string> arr = new List<string>();
                string phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                if (openid.Length == 11)
                {
                    phone = openid;
                }

                //为了兼容app通过微信支付领充openid,小马openid与微信openid兼容的问题 app openid为11位，在chargingdevice中找正在充电的设备 将两个opeid 关联
                if (!string.IsNullOrEmpty(phone))
                {
                    using (IDbConnection conn = WechatUtility.CreateDBConnection())
                    {
                        string sql = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                        openidList = conn.Query<string>(sql, conn).ToList();
                    }
                }
                openidList.Add(openid);
                openidList.Distinct();

                //市政府院内充电站设备无连接状态
                if (gunStatus == (int)EnumGunStatus.未插枪 && stationid == "6101120047")
                {
                    gunStatus = (int)EnumGunStatus.连接;
                }

                switch (gunStatus)
                {
                    case (int)Common.EnumGunStatus.连接:
                        if (ISVIN == true)
                        {
                            result.Add("code", 5);
                            result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                        }
                        else
                        {
                            result.Add("code", 1);
                            result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                        }
                        break;
                    case (int)Common.EnumGunStatus.充满:

                        result.Add("code", 1);
                        result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));

                        break;
                    case (int)Common.EnumGunStatus.启动中:
                    case (int)Common.EnumGunStatus.正在启动:

                        result.Add("code", -2);
                        result.Add("data", "启动中");

                        break;
                    case (int)Common.EnumGunStatus.充电中:
                        if (!openidList.Contains(redisOpenid))
                        {
                            result.Add("code", 0);
                            result.Add("data", "被占用");
                        }
                        else
                        {
                            result.Add("code", 99);
                            result.Add("data", "当前用户有未结束订单");
                        }
                        break;
                    case -99999:
                    case 99999:
                        result.Add("code", 0);
                        result.Add("data", "离线");
                        break;

                    default:
                        result.Add("code", 0);
                        result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                        break;
                }
                result.Add("paymentWay", (int)chargeLogic.GetPaymentMethod(stationid, ref payChannel, ref app));
                result.Add("app", (int)app);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetDeviceStatus。userToken={userToken},deviceCode={deviceCode}.", ex);
                result.RemoveAll();
                result.Add("code", 0);

                result.Add("error", ex.ToString());

            }
            return result;
        }
        /// <summary>
        /// 判定是否可以VIN启动
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <param name="userToken"></param>
        /// <returns></returns>
        public bool CheckVIN(string deviceCode,string userToken) {
            bool vin = false;
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            string stationid = WechatUtility.GetStationId(deviceCode);
            if (!string.IsNullOrEmpty(stationid))
            {
                List<string> VinPhone = redis.HashGet<List<string>>(RedisKey.VINInfo, stationid);
                string openid = WechatUtility.GetOpenIDByUserToken(userToken);

                string phone = redis.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis.HashGetString(RedisKey.UserAndPhone, openid);
                if (VinPhone != null && VinPhone.Contains(phone))
                {
                    vin = true;
                }
            }          
            return vin;
        }
        /// <summary>
        /// 获取设备状态
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns>1 可用;0 其他</returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject GetDeviceStatus(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                //LogEvent.WebAPILoger.Debug($"GetDeviceStatus 进入,userToken={userToken},deviceCode={deviceCode}");
                var chargeLogic = new ChargeLogic();
                var gunStatus = chargeLogic.GetDeviceStatus(userToken, deviceCode);

                if (gunStatus == 99)
                {
                    result.Add("code", 99);
                    result.Add("data", "当前用户有未结束订单");
                }
                else if (gunStatus == (int)Common.EnumGunStatus.连接 || gunStatus == (int)Common.EnumGunStatus.充满)
                {
                    result.Add("code", 1);
                    result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", Enum.GetName(typeof(Common.EnumGunStatus), gunStatus));
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetDeviceStatus。userToken={userToken},deviceCode={deviceCode}.", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// app侧停止设备，单设备模式
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="deviceCode">设备编码</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
      
        public JObject APPStopCharge(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var stopResult = chargeLogic.APPStopCharge(userToken, deviceCode);

                if (stopResult == true)
                {
                    result.Add("code", 1);
                    result.Add("data", "此处可能返回一些其他信息");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - APPStopCharge Error,userToken={userToken}, deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        /// <summary>
        /// app侧停止设备，多设备模式
        /// </summary>
        /// <param name="userToken">访问令牌</param>
        /// <param name="deviceCode">设备编码</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        [TokenFilter]
        public JObject APPStop(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                result = chargeLogic.APPStop(userToken, deviceCode);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - APPStopCharge Error,userToken={userToken}, deviceCode={deviceCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        ///// <summary>
        ///// 手动结束订单
        ///// </summary>
        ///// <param name="userToken"></param>
        ///// <returns></returns>
        //[HttpPost, HttpOptions]
        //public JObject ManualCloseOrder([FromBody]HistoryOrder order)
        //{
        //    LogEvent.WebAPILoger.Info($"ManualCloseOrder, order:{order.ObjectToString()}");
        //    JObject result = new JObject();
        //    try
        //    {
        //        var chargeLogic = new ChargeLogic();
        //        var close = chargeLogic.ManualCloseOrder(order);
        //        if (close)
        //        {
        //            result.Add("code", 1);
        //            result.Add("data", "关闭订单成功");
        //        }
        //        else
        //        {
        //            result.Add("code", 0);
        //            result.Add("data", "关闭订单失败");
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        LogEvent.WebAPILoger.Error("ManualCloseOrder", ex);
        //        result.Add("code", 0);
        //        result.Add("error", ex.ToString());
        //    }
        //    return result;
        //}

        /// <summary>
        /// 检测设备是否可用（连接及其特殊状态可用）
        /// </summary>
        /// <param name="deviceCode"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject CheckDeviceStatus(string deviceCode)
        {
            LogEvent.WebAPILoger.Info($"CheckDeviceStatus deviceCode={deviceCode}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                bool canUse = chargeLogic.CheckDeviceStatus(deviceCode);

                if (canUse)
                {
                    result.Add("code", 1);
                    result.Add("data", "设备可用");
                    LogEvent.WebAPILoger.Info($"CheckDeviceStatus deviceCode={deviceCode} 可用");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "设备不可用");
                    LogEvent.WebAPILoger.Info($"CheckDeviceStatus deviceCode={deviceCode} 不可用");
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - CheckDeviceStatus Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }

            return result;
        }

        /// <summary>
        /// 检测余额是否充足
        /// </summary>
        /// <param name="balance">用户当前余额（单位为厘）</param>
        /// <param name="chargingDeviceCount">已启动设备数</param>
        /// <param name="minimum">保底余额，即可启动时要求的最低余额（单位为厘）</param>
        /// <param name="increment">余额增量，即每多启动一个设备增加的最低余额（单位为厘）</param>
        /// <param name="incrementIncrement">增量增量，即每多启动一个设备增加的余额增量（单位为厘）</param>
        /// <example>
        /// minimum=1000, increment=1000, incrementIncrement=0
        /// chargingDeviceCount=0时，要求余额不小于1000；chargingDeviceCount=1时，要求余额不小于2000；
        /// chargingDeviceCount=2时，要求余额不小于3000；chargingDeviceCount=3时，要求余额不小于4000；
        /// 
        /// minimum=1000, increment=1000, incrementIncrement=500
        /// chargingDeviceCount=0时，要求余额不小于1000；chargingDeviceCount=1时，要求余额不小于2500；
        /// chargingDeviceCount=2时，要求余额不小于4500；chargingDeviceCount=3时，要求余额不小于7000；
        /// </example>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject CheckBalance(int balance, int chargingDeviceCount, int minimum = 1000, int increment = 1000, int incrementIncrement = 0)
        {
            LogEvent.WebAPILoger.Info($"CheckBalance balance={balance}, chargingDeviceCount={chargingDeviceCount}, minimum={minimum}, increment={increment}, incrementIncrement={incrementIncrement}");
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                bool enough = chargeLogic.CheckBalance(balance, chargingDeviceCount, minimum, increment, incrementIncrement);

                if (enough)
                {
                    result.Add("code", 1);
                    result.Add("data", "余额充足");
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "余额不足");
                }

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"OrdersController - CheckBalance Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());
            }
            return result;
        }

        #endregion
    }
}
